import { jsx } from 'react/jsx-runtime';
import { SIWEProvider } from 'starkwebkit';
import { getIronSession } from 'iron-session';
import { createPublicClient, http } from 'starkweb';
import * as allChains from 'starkweb/chains';
import { createSiwsMessage, generateSiwsNonce } from 'starkweb/siws';

const getSession = async (req, res, // ServerResponse<IncomingMessage>,
sessionConfig) => {
    const session = (await getIronSession(req, res, sessionConfig));
    return session;
};
const logoutRoute = async (req, res, sessionConfig, afterCallback) => {
    switch (req.method) {
        case 'GET':
            const session = await getSession(req, res, sessionConfig);
            session.destroy();
            if (afterCallback) {
                await afterCallback(req, res);
            }
            res.status(200).end();
            break;
        default:
            res.setHeader('Allow', ['GET']);
            res.status(405).end(`Method ${req.method} Not Allowed`);
    }
};
const nonceRoute = async (req, res, sessionConfig, afterCallback) => {
    switch (req.method) {
        case 'GET':
            const session = await getSession(req, res, sessionConfig);
            if (!session.nonce) {
                session.nonce = generateSiwsNonce();
                await session.save();
            }
            if (afterCallback) {
                await afterCallback(req, res, session);
            }
            res.send(session.nonce);
            break;
        default:
            res.setHeader('Allow', ['GET']);
            res.status(405).end(`Method ${req.method} Not Allowed`);
    }
};
const sessionRoute = async (req, res, sessionConfig, afterCallback) => {
    switch (req.method) {
        case 'GET':
            const session = await getSession(req, res, sessionConfig);
            if (afterCallback) {
                await afterCallback(req, res, session);
            }
            const { address, chainId } = session;
            res.send({ address, chainId });
            break;
        default:
            res.setHeader('Allow', ['GET']);
            res.status(405).end(`Method ${req.method} Not Allowed`);
    }
};
const verifyRoute = async (req, res, sessionConfig, config, afterCallback) => {
    var _a;
    switch (req.method) {
        case 'POST':
            // console.log('verifyRoute');
            try {
                const session = await getSession(req, res, sessionConfig);
                const { address, statement, nonce, chainId, version, domain, uri, signature } = req.body;
                // const parsed = parseSiwsMessage(message);
                if (nonce !== session.nonce) {
                    return res.status(422).end('Invalid nonce.');
                }
                let chain = (config === null || config === void 0 ? void 0 : config.chains)
                    ? Object.values(config.chains).find((c) => c.chain_id === chainId)
                    : undefined;
                if (!chain) {
                    // Try to find chain from allChains if not found in user-provided chains
                    chain = Object.values(allChains).find((c) => c.chain_id === chainId);
                }
                if (!chain) {
                    throw new Error('Chain not found.');
                }
                const publicClient = createPublicClient({
                    chain,
                    transport: http(),
                });
                // console.log('publicClient', signature);
                const verified = await publicClient.verifySiwsMessage({
                    statement,
                    signature,
                    nonce: (_a = session.nonce) !== null && _a !== void 0 ? _a : '1',
                    domain,
                    uri,
                    address,
                    chainId,
                    version,
                });
                // const verified = await publicClient.verifyMessage({
                //   message,
                //   signature,
                // });
                if (!verified) {
                    return res.status(422).end('Unable to verify signature.');
                }
                session.address = address;
                session.chainId = chainId;
                await session.save();
                if (afterCallback) {
                    await afterCallback(req, res, session);
                }
                res.status(200).end();
            }
            catch (error) {
                res.status(400).end(String(error));
            }
            break;
        default:
            res.setHeader('Allow', ['POST']);
            res.status(405).end(`Method ${req.method} Not Allowed`);
    }
};
const envVar = (name) => {
    const value = process.env[name];
    if (!value) {
        throw new Error(`Missing environment variable: ${name}`);
    }
    return value;
};
const configureServerSideSIWE = ({ config, session: { cookieName, password, cookieOptions, ...otherSessionOptions } = {}, options: { afterNonce, afterVerify, afterSession, afterLogout } = {}, }) => {
    const sessionConfig = {
        cookieName: cookieName !== null && cookieName !== void 0 ? cookieName : 'connectkit-next-siwe',
        password: password !== null && password !== void 0 ? password : envVar('SESSION_SECRET'),
        cookieOptions: {
            secure: process.env.NODE_ENV === 'production',
            ...(cookieOptions !== null && cookieOptions !== void 0 ? cookieOptions : {}),
        },
        ...otherSessionOptions,
    };
    const apiRouteHandler = async (req, res) => {
        if (!(req.query.route instanceof Array)) {
            throw new Error('Catch-all query param `route` not found. SIWE API page should be named `[...route].ts` and within your `pages/api` directory.');
        }
        const route = req.query.route.join('/');
        switch (route) {
            case 'nonce':
                return await nonceRoute(req, res, sessionConfig, afterNonce);
            case 'verify':
                return await verifyRoute(req, res, sessionConfig, config, afterVerify);
            case 'session':
                return await sessionRoute(req, res, sessionConfig, afterSession);
            case 'logout':
                return await logoutRoute(req, res, sessionConfig, afterLogout);
            default:
                return res.status(404).end();
        }
    };
    return {
        apiRouteHandler,
        getSession: async (req, res) => await getSession(req, res, sessionConfig),
    };
};
const configureClientSIWE = ({ apiRoutePrefix, statement = 'Sign In With Starknet.', }) => {
    const NextSIWEProvider = (props) => {
        return (jsx(SIWEProvider, { getNonce: async () => {
                const res = await fetch(`${apiRoutePrefix}/nonce`);
                if (!res.ok) {
                    throw new Error('Failed to fetch SIWE nonce');
                }
                const nonce = await res.text();
                return nonce;
            }, createMessage: async ({ nonce, address, chainId }) => {
                return createSiwsMessage({
                    version: '1',
                    domain: window.location.host,
                    uri: window.location.origin,
                    address,
                    chainId,
                    nonce,
                    statement,
                }).toString();
            }, verifyMessage: ({ address, statement, nonce, chainId, version, domain, uri, signature }) => {
                // console.log('verifyMessage', signature);
                // console.log('verifyMessage', address, statement, nonce, chainId, version, domain, uri, signature);
                return fetch(`${apiRoutePrefix}/verify`, {
                    method: 'POST',
                    headers: {
                        'Content-Type': 'application/json',
                    },
                    body: JSON.stringify({ address, statement, nonce, chainId, version, domain, uri, signature }),
                }).then((res) => res.ok);
            }, getSession: async () => {
                const res = await fetch(`${apiRoutePrefix}/session`);
                if (!res.ok) {
                    throw new Error('Failed to fetch SIWE session');
                }
                const { address, chainId } = await res.json();
                return address && chainId ? { address, chainId } : null;
            }, signOut: () => fetch(`${apiRoutePrefix}/logout`).then((res) => res.ok), ...props }));
    };
    return {
        Provider: NextSIWEProvider,
    };
};

export { configureClientSIWE, configureServerSideSIWE };
//# sourceMappingURL=index.es.js.map
